/*
 *	win.c - window manager
 */

#include "filer.h"

static RECT	rcWin;
static RECT	rcTitle;
static RECT	rcPage;
static RECT	rcSep;
static RECT	rcClient;

/*
 *	CreateWinRect
 */
static void CreateWinRect(PCREATEWIN pcw)
{
	PSCRNOBJ	pso;
	int		wsmall, hsmall;
	int		wpage;

	pso = GetScrnObject();

	XfontSelectFontObject(FO_SMALL);
	wsmall = XfontGetMaxCharWidth(' ');
	hsmall = XfontGetCharHeight();

	// Window rect

	rcWin.x = XGAP_WINDOW;
	rcWin.y = YGAP_WINDOW;
	rcWin.w = GetScrnWidth(pso) - XGAP_WINDOW * 2;
	rcWin.h = GetScrnHeight(pso) - YGAP_WINDOW * 2;

	// Compute width for "(00000/00000)"

	wpage = XfontGetStrWidth("(/)");
	{
		u8	*p;
		int	maxw, w;

		p = "0123456789";
		maxw = 0;
		w = 0;
		while (*p) {
			w = XfontGetCharWidth(&p);
			if (w > maxw)
				maxw = w;
		}
		wpage += w * 10;
	}

	// Page rect

	rcPage.x = rcWin.x + (rcWin.w - wpage);
	rcPage.y = rcWin.y;
	rcPage.w = wpage;
	rcPage.h = hsmall;

	// Title rect

	rcTitle.x = rcWin.x + pcw->xStartTitle;
	rcTitle.y = rcWin.y;
	rcTitle.w = rcWin.w - rcPage.w - pcw->xStartTitle;
	rcTitle.h = hsmall;

	// Separator rect

	rcSep.x = rcWin.x;
	rcSep.y = rcWin.y + hsmall;
	rcSep.w = rcWin.w;
	rcSep.h = 1;

	// Client rect

	rcClient.x = rcWin.x;
	rcClient.y = rcSep.y + rcSep.h + 1;
	rcClient.w = rcWin.w;
	rcClient.h = rcWin.h - (rcClient.y - rcWin.y);
}

/*
 *	GetTitleRect
 */
void GetTitleRect(RECT *prc)
{
	*prc = rcTitle;
}

/*
 *	SetWinTitle
 */
void SetWinTitle(u8 *ptitle)
{
	XfontSelectFontObject(FO_SMALL);
	LeftPrintf(rcTitle.x, rcTitle.y, rcTitle.w, "%s", ptitle);
}

/*
 *	GetPageRect
 */
void GetPageRect(RECT *prc)
{
	*prc = rcPage;
}

/*
 *	ShowWinPages
 */
void ShowWinPages(int page, int npages)
{
	if (npages > 1) {
		XfontSelectFontObject(FO_SMALL);
		RightPrintf(rcPage.x, rcPage.y, rcPage.w, "(%d/%d)",
			page, npages);
	} else
		EraseScrn(rcPage.x, rcPage.y, rcPage.w, rcPage.h);
}

/*
 *	DrawSeparator
 */
static void DrawSeparator()
{
	FillScrn(rcSep.x, rcSep.y, rcSep.w, rcSep.h, RGB(15,15,15));
}

/*
 *	CreateWinFrame
 */
void CreateWinFrame(PCREATEWIN pcw)
{
	CreateWinRect(pcw);
	ClearScrn();
	DrawSeparator();
}

/*
 *	GetClientRect
 */
void GetClientRect(PRECT prc)
{
	*prc = rcClient;
}

/*
 *	GetWindowRect
 */
void GetWindowRect(PRECT prc)
{
	*prc = rcWin;
}

/*
 *	SetWinMode
 */
void SetWinMode(int mode)
{
	SetScrnMode(mode);
	SetKeyMap(mode);
}

