/*
 *	scrn.c
 */

#include <scrn.h>

#define VRAM_END	((vu16 *)VRAM_ADDR + SCREEN_WIDTH * SCREEN_HEIGHT)

static u16	fgColor, bgColor;

/*
 *	ClearScrn
 */
void ClearScrn()
{
	vu16		*vp;
	PSCRNOBJ	pso;

	pso = GetScrnObject();

	vp = (vu16 *)VRAM_ADDR; 
	while (vp < VRAM_END)
		*vp++ = bgColor;
}

/*
 *	FillScrn
 */
void FillScrn(int x, int y, int width, int height, u16 c)
{
	vu16		*vp0, *vp;
	int		i;
	PSCRNOBJ	pso;
	int		xd, yd;

	pso = GetScrnObject();
	vp0 = pso->ScrnPosToVram(x, y);
	xd = GetScrnNextX(pso);
	yd = GetScrnNextY(pso);

	while (height-- > 0) {
		vp = vp0;
		for (i = 0; i < width; i++) {
			*vp = c;
			vp += xd;
		}
		vp0 += yd;
	}
}

/*
 *	BoxScrn
 */
void BoxScrn(int x, int y, int width, int height, u16 c)
{
	vu16		*vp0, *vp;
	int		i;
	PSCRNOBJ	pso;
	int		xd, yd;

	pso = GetScrnObject();
	vp0 = pso->ScrnPosToVram(x, y);
	xd = GetScrnNextX(pso);
	yd = GetScrnNextY(pso);

	if (height > 0) {
		FillScrn(x, y, width, 1, c);
		y++;
		height--;
	}

	vp0 = pso->ScrnPosToVram(x, y);
	while (height > 1) {
		*vp0 = c;
		vp = vp0 + (xd * (width - 1));
		*vp = c;

		vp0 += yd;
		y++;
		height--;
	}

	if (height > 0)
		FillScrn(x, y, width, 1, c);
}

/*
 *	EraseScrn
 */
void EraseScrn(int x, int y, int width, int height)
{
	PSCRNOBJ	pso;

	pso = GetScrnObject();
	FillScrn(x, y, width, height, bgColor);
}

/*
 *	ReverseScrn
 */
void ReverseScrn(int x, int y, int width, int height)
{
	vu16		*vp0, *vp;
	int		i;
	PSCRNOBJ	pso;
	int		xd, yd;

	pso = GetScrnObject();
	vp0 = pso->ScrnPosToVram(x, y);
	xd = GetScrnNextX(pso);
	yd = GetScrnNextY(pso);

	while (height-- > 0) {
		vp = vp0;
		for (i = 0; i < width; i++) {
			*vp ^= 0x7fff;
			vp += xd;
		}
		vp0 += yd;
	}
}

/*
 *	CopyBitmap16
 */
void CopyBitmap16(PBITMAP pbm, int x, int y, u16 transparent)
{
	vu16		*pv, *pv0;
	const u16	*pi;
	int		w, h;
	PSCRNOBJ	pso;
	int		xd, yd;
	u16		color;

	pso = GetScrnObject();
	xd = GetScrnNextX(pso);
	yd = GetScrnNextY(pso);

	pv0 = pso->ScrnPosToVram(x, y);
	pi = pbm->pimage;
	h = pbm->height;
	while (h-- > 0) {
		pv = pv0;
		w = pbm->width;
		while (w-- > 0) {
			color = (*pi == transparent) ? bgColor : *pi;
			pi++;
			*pv = color;
			pv += xd;
		}
		pv0 += yd;
	}
}

/*
 *	SetScrnColor
 */
void SetScrnColor(u16 fgcolor, u16 bgcolor)
{
	fgColor = fgcolor;
	bgColor = bgcolor;
}

/*
 *	iwGetScrnColor
 */
attriwram
void iwGetScrnColor(u16 *pfgcolor, u16 *pbgcolor)
{
	*pfgcolor = fgColor;
	*pbgcolor = bgColor;
}

PSCRNOBJ (*GetScrnObject)();
void (*GetScrnColor)(u16 *pfgcolor, u16 *pbgcolor);

/*
 *	ScrnInit
 */
void ScrnInit()
{
	GetScrnObject = iwGetScrnObject;
	GetScrnColor = iwGetScrnColor;

	WRITE_REG16(LCD_CTL, 0x0403);	// BG2:Mode3
}

