/*
 *	xfontobj.c
 */

#include <xfont.h>
#include <chartype.h>

#define HASKANA(pxf)	((pxf)->ecode >= 0x80)

static XFONTOBJ		xfontobj[FO_LAST][CT_LAST];
static PXFONTOBJ	pCurXfontobj;

/*
 *	getcode1
 */
attriwram
static int getcode1(u8 **ppcode)
{
	if (**ppcode == 0x09) {	// TAB code
		(*ppcode)++;
		return ' ';
	}
	return *(*ppcode)++;
}

/*
 *	getcode2
 */
attriwram
static int getcode2(u8 **ppcode)
{
	u8		l, h;

	h = *(*ppcode)++;
	l = *(*ppcode)++;
	return MAKEWORD(l, h);
}

/*
 *	SBCS Kana to DBCS Kana mapper
 */

static const u16	KanaDaku1[] = {
	0x834b, 0x834d, 0x834f, 0x8351, 0x8353, // GaGiGuGeGo
	0x8355, 0x8357, 0x8359, 0x835b, 0x835d,	// ZaZiZuZeZo
	0x835f, 0x8361, 0x8364, 0x8366, 0x8368, // DaDiDuDeDo
};

static const u16	KanaDaku2[] = {
	0x836f, 0x8372, 0x8375, 0x8378, 0x837b, // BaBiBuBeBo
};

static const u16	KanaSound[] = {
	0x8370, 0x8373, 0x8376, 0x8379, 0x837c, // PaPiPuPePo
};

/*
 *	getcodekana
 */
attriwram
static int getcodekana(u8 **ppcode)
{
	u8		c1, c2;

	c1 = *(*ppcode)++;
	c2 = *(*ppcode);

	switch (c2) {
	default:
		return c1;
	case 0xde:	// Daku-ten
		if (c1 >= 0xb6 && c1 <= 0xc4) {
			(*ppcode)++;
			return KanaDaku1[c1 - 0xb6];
		} else if (c1 >= 0xca && c1 <= 0xce) {
			(*ppcode)++;
			return KanaDaku2[c1 - 0xca];
		}
		return c1;
	case 0xdf:	// Semi Voice-mark
		if (c1 >= 0xca && c1 <= 0xce) {
			(*ppcode)++;
			return KanaSound[c1 - 0xca];
		}
		return c1;
	}
}

/*
 *	XfontSetFontObject
 */
void XfontSetFontObject(int index, PXFONTHEAD pxa, PXFONTHEAD pxk)
{
	PXFONTOBJ	pxo;

	if (index < 0 || index >= FO_LAST)
		return;

	//	Ansi
	pxo = &xfontobj[index][CT_ANSI];
	pxo->pxf = pxa;
	pxo->code2index = XfontAnsiCode2Index;
	pxo->getcode = getcode1;

	//	Kana
	pxo = &xfontobj[index][CT_KANA];
	if (HASKANA(pxa)) {
		pxo->pxf = pxa;
		pxo->code2index = XfontAnsiCode2Index;
		pxo->getcode = getcode1;
	} else {
		pxo->pxf = pxk;
		pxo->code2index = XfontKanaCode2Index;
		pxo->getcode = getcodekana;
	}

	//	Kanji
	pxo = &xfontobj[index][CT_KANJI];
	pxo->pxf = pxk;
	pxo->code2index = XfontKanjiCode2Index;
	pxo->getcode = getcode2;

	//	CCT
	pxo->pcct = XfontGetCCT(pxk->cct);
}

/*
 *	XfontSelectFontObject
 */
void XfontSelectFontObject(int index)
{
	if (index < 0 || index >= FO_LAST)
		return;

	pCurXfontobj = &xfontobj[index][0];
}

/*
 *	XfontGetFontObject
 */
attriwram
PXFONTOBJ iwXfontGetFontObject(u8 code)
{
	return &pCurXfontobj[GETCHARTYPE(code)];
}

